using System;

using CMS.GlobalHelper;
using CMS.Ecommerce;
using CMS.EcommerceProvider;
using CMS.EventLog;
using CMS.UIControls;

public partial class CMSModules_Ecommerce_CMSPages_WorldPayPN : CMSPage
{
    private int orderId = 0;
    private string transactionId = "";
    private string paymentStatus = "";
    private string orderCulture = null;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Get order id
        orderId = ValidationHelper.GetInteger(Request.Form["M_orderId"], 0);

        // Get transaction id
        transactionId = ValidationHelper.GetString(Request.Form["transId"], "");

        // Get payment status
        paymentStatus = ValidationHelper.GetString(Request.Form["transStatus"], "");

        // Get notification culture stored in custom field
        orderCulture = ValidationHelper.GetString(Request.Form["M_culture"], "");

        WorldPayProvider worldPayProvider = null;
        string errorMessage = "";

        // Get WorldPay provider
        OrderInfo oi = OrderInfoProvider.GetOrderInfo(orderId);
        if (oi != null)
        {
            try
            {
                worldPayProvider = (WorldPayProvider)CMSPaymentGatewayProvider.GetPaymentGatewayProvider(oi.OrderPaymentOptionID);
                worldPayProvider.OrderId = orderId;
            }
            catch (Exception ex)
            {
                // Log exception
                errorMessage = EventLogProvider.GetExceptionLogMessage(ex);
                LogEvent(errorMessage, "Payment_Provider_Not_Found");
                return;
            }
        }
        else
        {
            // Order not found
            errorMessage = string.Format(GetString("PaymentGatewayProvider.ordernotfound"), orderId);
            LogEvent(errorMessage, "Order_Not_Found");
            return;
        }

        WorldPayPaymentResultInfo worldPPayResult = (WorldPayPaymentResultInfo)(worldPayProvider.PaymentResult);

        // Set payment as verified / not verified
        worldPPayResult.WorldPayPaymentVerified = (paymentStatus.ToLower() == "y");

        // Set transaction ID
        worldPPayResult.PaymentTransactionID = transactionId;

        // Set payment status (take received status seriously only if payment was verified)
        worldPPayResult.WorldPayPaymentStatus = WorldPayPaymentResultInfo.GetWorldPayPaymentStatus(worldPPayResult.WorldPayPaymentVerified ? paymentStatus : "c");

        // Set payment as completed / not completed
        switch (worldPPayResult.WorldPayPaymentStatus)
        {
            case WorldPayPaymentStatusEnum.Completed:
                worldPPayResult.PaymentIsCompleted = true;
                break;

            case WorldPayPaymentStatusEnum.Failed:
                worldPPayResult.PaymentIsCompleted = false;
                break;

            default:
                worldPPayResult.PaymentIsCompleted = false;
                break;
        }

        // Compare payment data to order data
        string error = worldPayProvider.ComparePaymentDataToOrderData(Request.Form);
        if (error != "")
        {
            worldPPayResult.PaymentIsCompleted = false;
            worldPPayResult.WorldPayPaymentVerified = false;
            worldPPayResult.PaymentDescription = error;
        }

        // Order culture
        worldPayProvider.ShoppingCartInfoObj.ShoppingCartCulture = orderCulture;

        // Update order payment result in database
        errorMessage = worldPayProvider.UpdateOrderPaymentResult();
        if (errorMessage != "")
        {
            // Log the event
            errorMessage += error;
            LogEvent(errorMessage, "Order_Payment_Result_Update");
        }
    }


    /// <summary>
    /// Logs error.
    /// </summary>
    /// <param name="message">Error message</param>
    /// <param name="eventCode">Event code</param>
    protected void LogEvent(string message, string eventCode)
    {
        try
        {
            // Add some additional information to the error message            
            string info = "ORDER ID: " + orderId + "\r\n";
            info += "TRANSACTION ID: " + transactionId + "\r\n";
            info += "PAYMENT STATUS: " + paymentStatus + "\r\n";

            message = info + message;

            EventLogProvider evenLog = new EventLogProvider();
            evenLog.LogEvent(EventLogProvider.EVENT_TYPE_ERROR, DateTime.Now, "WorldPay PN", eventCode, 0, "", 0, "", "", message, 0, "");
        }
        catch
        {
            // Unable to log the event
        }
    }
}

